/*
    File:             kunzel.cpp
    Author:           Jiri Madera
    Purpose:          c
    Source:           Kunzel, 19xx;
    Assumptions:


*/


#include "richards.h"
#include "globalt.h"


richards::richards()
{
  //  parameters of van Genuchten model
  alpha=0.0;
  n=0.0;
  m=0.0;
  
  //  saturated hydraulic conductivity
  kks=0.0;
  //  saturated water content
  thetas=0.0;
  //  residual water content
  thetar=0.0;
  
}

richards::~richards()
{

}

/**
   function reads material parameters
   
   @param in - input file
   
   1.12.2011
*/
void richards::read(XFILE *in)
{
  //  parameters of van Genuchten model
  //  saturated hydraulic conductivity
  //  saturated water content
  //  residual water content
  //  specific storage
  xfscanf (in,"%lf %lf %lf %lf %lf %lf %lf",&alpha,&n,&m,&kks,&thetas,&thetar,&storage);
}

/**
   function prints material parameters
   
   @param out - output file
   
   1.12.2011
*/
void richards::print(FILE *out)
{
  //  parameters of van Genuchten model
  //  saturated hydraulic conductivity
  //  saturated water content
  //  residual water content
  fprintf (out,"%lf %lf %lf %lf %lf %lf\n",alpha,n,m,kks,thetas,thetar);
}

/**
   function computes hydraulic conductivity
   
   @param h - hydraulic head
   
   1.12.2011
*/
double richards::kk_value (double h)
{
  if (h < 0){
    return kks*(pow(1 - pow(-(alpha*h),m*n)/pow(1 + pow(-(alpha*h),n),m),2)/pow(1 + pow(-(alpha*h),n),m/2.0));
  }
  else{
    return kks;
  }

}

/**
   function computes the derivative of K with respect to h (dK/dh van Genuchten).
   
   @param h - hydraulic head
   
   1.12.2011
*/
double richards::dkkdh_value (double h)
{
  
  if (h < 0){
    return kks*((alpha*pow(-(alpha*h),-1 + n)*pow(1 + pow(-(alpha*h),n),-1 - m/2.)*pow(1 - pow(-(alpha*h),m*n)/pow(1 + pow(-(alpha*h),n),m),2)*m*n)/2. + (2*(1 - pow(-(alpha*h),m*n)/pow(1 + pow(-(alpha*h),n),m))*(-(alpha*pow(-(alpha*h),-1 + n + m*n)*pow(1 + pow(-(alpha*h),n),-1 - m)*m*n) + (alpha*pow(-(alpha*h),-1 + m*n)*m*n)/pow(1 + pow(-(alpha*h),n),m)))/pow(1 + pow(-(alpha*h),n),m/2.));
  }
  else{
    return 0.0;
  }
  
}


/**
   function computes second derivative of K with respect to h (d^2K/dh^2 van Genuchten).
   
   @param h - hydraulic head
   
   1.12.2011
*/
double richards::ddkkddh_value (double h)
{
  if (h < 0){
    return kks*( -(pow(alpha,2)*pow(-(alpha*h),-2 + n)*pow(1 + pow(-(alpha*h),n),-1 - m/2.)*
	  pow(1 - pow(-(alpha*h),m*n)/
	  pow(1 + pow(-(alpha*h),n),m),2)*m*(-1 + n)*n)/2.0
	    - (pow(alpha,2)*pow(-(alpha*h),-2 + 2*n)*
	    pow(1 + pow(-(alpha*h),n),-2 - m/2.)*
	  pow(1 - pow(-(alpha*h),m*n)/
	      pow(1 + pow(-(alpha*h),n),m),2)*(-1 - m/2.)*m*
	  pow(n,2))/2. + 2*alpha*pow(-(alpha*h),-1 + n)*
	    pow(1 + pow(-(alpha*h),n),-1 - m/2.)*
	  (1 - pow(-(alpha*h),m*n)/
	    pow(1 + pow(-(alpha*h),n),m))*m*n*
	  (-(alpha*pow(-(alpha*h),-1 + n + m*n)*
	    pow(1 + pow(-(alpha*h),n),-1 - m)*m*n) + 
	  (alpha*pow(-(alpha*h),-1 + m*n)*m*n)/
	  pow(1 + pow(-(alpha*h),n),m)) + 
	  (2*pow(-(alpha*pow(-(alpha*h),-1 + n + m*n)*
	      pow(1 + pow(-(alpha*h),n),-1 - m)*m*n) + 
	    (alpha*pow(-(alpha*h),-1 + m*n)*m*n)/
	    pow(1 + pow(-(alpha*h),n),m),2))/
	    pow(1 + pow(-(alpha*h),n),m/2.) + 
	  (2*(1 - pow(-(alpha*h),m*n)/
	    pow(1 + pow(-(alpha*h),n),m))*
	    (pow(alpha,2)*pow(-(alpha*h),-2 + 2*n + m*n)*
	    pow(1 + pow(-(alpha*h),n),-2 - m)*(-1 - m)*m*
	    pow(n,2) + pow(alpha,2)*
	    pow(-(alpha*h),-2 + n + m*n)*
	    pow(1 + pow(-(alpha*h),n),-1 - m)*pow(m,2)*
	    pow(n,2) - (pow(alpha,2)*
	      pow(-(alpha*h),-2 + m*n)*m*n*(-1 + m*n))/
	    pow(1 + pow(-(alpha*h),n),m) + 
	    pow(alpha,2)*pow(-(alpha*h),-2 + n + m*n)*
	    pow(1 + pow(-(alpha*h),n),-1 - m)*m*n*
	    (-1 + n + m*n)))/pow(1 + pow(-(alpha*h),n),m/2.)) ;
  }
  else{
    return 0.0;
  }

}

/**
   function computes capacity (van Genuchten).
   
   @param h - hydraulic head
   
   1.12.2011
*/
double richards::c_value (double h)
{
  if (h < 0){
    return alpha*pow(-(alpha*h),-1 + n)*pow(1 + pow(-(alpha*h),n),-1 - m)*m*n*(thetas - thetar) + 
      (storage*((thetas - thetar)/pow(1 + pow(-(alpha*h),n),m) + 
		thetar))/thetas;
  }
  else{
    return storage;
  }
}



/**
   function computes derivative of capacity with respect to hydraulic head (van Genuchten).
   
   @param h - hydraulic head
   
   1.12.2011
*/
double richards::dcdh_value (double h)
{
  if (h < 0){
    return -(pow(alpha,2)*pow(-(alpha*h),-2 + n)*
	     pow(1 + pow(-(alpha*h),n),-1 - m)*m*(-1 + n)*n*
	     (thetas - thetar)) - pow(alpha,2)*pow(-(alpha*h),-2 + 2*n)*
      pow(1 + pow(-(alpha*h),n),-2 - m)*(-1 - m)*m*pow(n,2)*
      (thetas - thetar) + (alpha*pow(-(alpha*h),-1 + n)*
			     pow(1 + pow(-(alpha*h),n),-1 - m)*m*n*storage*
			     (thetas - thetar))/thetas;
  }
  else{
    return 0.0; 
  }
}


/**
   function computes conductivity %matrix of the material
   in the required integration point
   
   @param d - conductivity %matrix of material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point

*/
void richards::matcond (matrix &d,long ri,long ci,long ipp)
{
  long n;
  n = d.n;

  switch (n){
  case 1:{
    matcond1d (d,ri,ci,ipp);//1D
    break;
  }
  case 2:{
    matcond2d (d,ri,ci,ipp);//2D
    break;
  }
  case 3:{
    matcond3d (d,ri,ci,ipp);//3D
    break;
  }
  default:{
    print_err("unknown number of flux components is required",__FILE__,__LINE__,__func__);
  }
  }
}


/**
   function creates conductivity %matrix of the material for 1D problems

   @param d   - conductivity %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
*/
void richards::matcond1d (matrix &d,long ri,long ci,long ipp)
{
  double h,k;
  
  //  actual hydraulic head
  h = Tm->ip[ipp].av[0];
  
  //  hydraulic conductivity
  k = kk_value (h);
  
  d[0][0] = k;
}

/**
   function creates conductivity %matrix of the material for 2D problems

   @param d   - conductivity %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
*/
void richards::matcond2d (matrix &d,long ri,long ci,long ipp)
{
  double h,k;
  
  //  actual hydraulic head
  h = Tm->ip[ipp].av[0];
  
  //  hydraulic conductivity
  k = kk_value (h);
  
  fillm(0.0,d);
  
  d[0][0] = k;   d[0][1] = 0.0;
  d[1][0] = 0.0; d[1][1] = k;
  //fprintf (stdout,"\n vodivost  %le",d[0][0]);

}

/**
   function creates conductivity %matrix of the material for 3D problems

   @param d   - conductivity %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
*/

void richards::matcond3d (matrix &d,long ri,long ci,long ipp)
{
  double h,k;
  
  //  actual hydraulic head
  h = Tm->ip[ipp].av[0];
  
  //  hydraulic conductivity
  k = kk_value (h);
  
  fillm(0.0,d);
  
  d[0][0]=k;   d[0][1]=0.0; d[0][2]=0.0;
  d[1][0]=0.0; d[1][1]=k;   d[1][2]=0.0;
  d[2][0]=0.0; d[2][1]=0.0; d[2][2]=k;
}

void richards::matcond2 (matrix &d,long ri,long ci,long ipp)
{
  long n;
  n = d.n;
  
  switch (n){
  case 1:{
    matcond1d_2 (d,ri,ci,ipp);//1D
    break;
  }
  case 2:{
    matcond2d_2 (d,ri,ci,ipp);//2D
    break;
  }
  case 3:{
    matcond3d_2 (d,ri,ci,ipp);//3D
    break;
  }
  default:{
    print_err("unknown number of components is required",__FILE__,__LINE__,__func__);
  }
  }
}

/**
   function creates conductivity %matrix of the material for 1D problems (convective term)
   
   this %matrix is correct only for homogeneous material on finite elements
   
   @param d   - conductivity %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
*/
void richards::matcond1d_2 (matrix &d,long ri,long ci,long ipp)
{
  double h;
  
  //  actual hydraulic head
  h = Tm->ip[ipp].av[0];
  
  fillm(0.0,d);
  d[0][0]=0.0-dkkdh_value (h);
}


/**
   function creates conductivity %matrix of the material for 1D problems (convective term)

   this %matrix is correct only for homogeneous material on finite elements
   
   @param d   - conductivity %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
   
*/
void richards::matcond2d_2 (matrix &d,long ri,long ci,long ipp)
{
  double h;
  
  //  actual hydraulic head
  h = Tm->ip[ipp].av[0];
  
  fillm(0.0,d);
  d[0][0]=0.0;
  d[0][1]=0.0-dkkdh_value (h);
  
  //fprintf (stdout,"\n konvekce %le",d[0][1]);
}

/**
   function creates conductivity %matrix of the material for 1D problems (convective term)

   this %matrix is correct only for homogeneous material on finite elements
   
   @param d   - conductivity %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
*/
void richards::matcond3d_2 (matrix &d,long ri,long ci,long ipp)
{
  double h;
  
  //  actual hydraulic head
  h = Tm->ip[ipp].av[0];
  
  fillm(0.0,d);
  d[0][0]=0.0;
  d[0][1]=0.0;
  d[0][2]=0.0-dkkdh_value (h);
}


/**
   function creates capacity %matrix of the material

   @param c   - capacity %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
*/
void richards::matcap (double &c,long ri,long ci,long ipp)
{
  double h;
  
  //  actual hydraulic head
  h = Tm->ip[ipp].av[0];
  
  //  capacity
  c = c_value (h);

}




